@@ -1,12 +1,17 @@
 package main
 
 import (
+	"io/fs"
 	"math"
+	"os"
 	"reflect"
+	"strings"
 	"testing"
 
+	"github.com/restic/restic/internal/errors"
 	"github.com/restic/restic/internal/restic"
 	rtest "github.com/restic/restic/internal/test"
+	"github.com/restic/restic/internal/ui/progress"
 )
 
 func TestParsePercentage(t *testing.T) {
@@ -71,7 +76,7 @@ func TestSelectPacksByBucket(t *testing.T) {
 	var testPacks = make(map[restic.ID]int64)
 	for i := 1; i <= 10; i++ {
 		id := restic.NewRandomID()
-		// ensure relevant part of generated id is reproducable
+		// ensure relevant part of generated id is reproducible
 		id[0] = byte(i)
 		testPacks[id] = 0
 	}
@@ -124,7 +129,7 @@ func TestSelectRandomPacksByPercentage(t *testing.T) {
 }
 
 func TestSelectNoRandomPacksByPercentage(t *testing.T) {
-	// that the a repository without pack files works
+	// that the repository without pack files works
 	var testPacks = make(map[restic.ID]int64)
 	selectedPacks := selectRandomPacksByPercentage(testPacks, 10.0)
 	rtest.Assert(t, len(selectedPacks) == 0, "Expected 0 selected packs")
@@ -158,8 +163,84 @@ func TestSelectRandomPacksByFileSize(t *testing.T) {
 }
 
 func TestSelectNoRandomPacksByFileSize(t *testing.T) {
-	// that the a repository without pack files works
+	// that the repository without pack files works
 	var testPacks = make(map[restic.ID]int64)
 	selectedPacks := selectRandomPacksByFileSize(testPacks, 10, 500)
 	rtest.Assert(t, len(selectedPacks) == 0, "Expected 0 selected packs")
 }
+
+func checkIfFileWithSimilarNameExists(files []fs.DirEntry, fileName string) bool {
+	found := false
+	for _, file := range files {
+		if file.IsDir() {
+			dirName := file.Name()
+			if strings.Contains(dirName, fileName) {
+				found = true
+			}
+		}
+	}
+	return found
+}
+
+func TestPrepareCheckCache(t *testing.T) {
+	// Create a temporary directory for the cache
+	tmpDirBase := t.TempDir()
+
+	testCases := []struct {
+		opts           CheckOptions
+		withValidCache bool
+	}{
+		{CheckOptions{WithCache: true}, true},   // Shouldn't create temp directory
+		{CheckOptions{WithCache: false}, true},  // Should create temp directory
+		{CheckOptions{WithCache: false}, false}, // Should create cache directory first, then temp directory
+	}
+
+	for _, testCase := range testCases {
+		t.Run("", func(t *testing.T) {
+			if !testCase.withValidCache {
+				// remove tmpDirBase to simulate non-existing cache directory
+				err := os.Remove(tmpDirBase)
+				rtest.OK(t, err)
+			}
+			gopts := GlobalOptions{CacheDir: tmpDirBase}
+			cleanup := prepareCheckCache(testCase.opts, &gopts, &progress.NoopPrinter{})
+			files, err := os.ReadDir(tmpDirBase)
+			rtest.OK(t, err)
+
+			if !testCase.opts.WithCache {
+				// If using a temporary cache directory, the cache directory should exist
+				// listing all directories inside tmpDirBase (cacheDir)
+				// one directory should be tmpDir created by prepareCheckCache with 'restic-check-cache-' in path
+				found := checkIfFileWithSimilarNameExists(files, "restic-check-cache-")
+				if !found {
+					t.Errorf("Expected temporary directory to exist, but it does not")
+				}
+			} else {
+				// If not using the cache, the temp directory should not exist
+				rtest.Assert(t, len(files) == 0, "expected cache directory not to exist, but it does: %v", files)
+			}
+
+			// Call the cleanup function to remove the temporary cache directory
+			cleanup()
+
+			// Verify that the cache directory has been removed
+			files, err = os.ReadDir(tmpDirBase)
+			rtest.OK(t, err)
+			rtest.Assert(t, len(files) == 0, "Expected cache directory to be removed, but it still exists: %v", files)
+		})
+	}
+}
+
+func TestPrepareDefaultCheckCache(t *testing.T) {
+	gopts := GlobalOptions{CacheDir: ""}
+	cleanup := prepareCheckCache(CheckOptions{}, &gopts, &progress.NoopPrinter{})
+	_, err := os.ReadDir(gopts.CacheDir)
+	rtest.OK(t, err)
+
+	// Call the cleanup function to remove the temporary cache directory
+	cleanup()
+
+	// Verify that the cache directory has been removed
+	_, err = os.ReadDir(gopts.CacheDir)
+	rtest.Assert(t, errors.Is(err, os.ErrNotExist), "Expected cache directory to be removed, but it still exists")
+}
