@@ -5,11 +5,13 @@ import (
 	"io"
 	"os"
 	"path"
+	"slices"
 	"sync"
 	"syscall"
 	"time"
 
 	"github.com/restic/restic/internal/errors"
+	"github.com/restic/restic/internal/restic"
 )
 
 // Reader is a file system which provides a directory with a single file. When
@@ -39,14 +41,21 @@ func (fs *Reader) VolumeName(_ string) string {
 	return ""
 }
 
-// MapFilename is a temporary hack to prepare a filename for usage with
-// NodeFromFileInfo. This is only relevant for LocalVss.
-func (fs *Reader) MapFilename(filename string) string {
-	return filename
+func (fs *Reader) fi() *ExtendedFileInfo {
+	return &ExtendedFileInfo{
+		Name:    fs.Name,
+		Mode:    fs.Mode,
+		ModTime: fs.ModTime,
+		Size:    fs.Size,
+	}
 }
 
-// Open opens a file for reading.
-func (fs *Reader) Open(name string) (f File, err error) {
+func (fs *Reader) OpenFile(name string, flag int, _ bool) (f File, err error) {
+	if flag & ^(O_RDONLY|O_NOFOLLOW) != 0 {
+		return nil, pathError("open", name,
+			fmt.Errorf("invalid combination of flags 0x%x", flag))
+	}
+
 	switch name {
 	case fs.Name:
 		fs.open.Do(func() {
@@ -60,7 +69,7 @@ func (fs *Reader) Open(name string) (f File, err error) {
 		return f, nil
 	case "/", ".":
 		f = fakeDir{
-			entries: []os.FileInfo{fs.fi()},
+			entries: []string{fs.fi().Name},
 		}
 		return f, nil
 	}
@@ -68,56 +77,18 @@ func (fs *Reader) Open(name string) (f File, err error) {
 	return nil, pathError("open", name, syscall.ENOENT)
 }
 
-func (fs *Reader) fi() os.FileInfo {
-	return fakeFileInfo{
-		name:    fs.Name,
-		size:    fs.Size,
-		mode:    fs.Mode,
-		modtime: fs.ModTime,
-	}
-}
-
-// OpenFile is the generalized open call; most users will use Open
-// or Create instead.  It opens the named file with specified flag
-// (O_RDONLY etc.) and perm, (0666 etc.) if applicable.  If successful,
-// methods on the returned File can be used for I/O.
-// If there is an error, it will be of type *os.PathError.
-func (fs *Reader) OpenFile(name string, flag int, _ os.FileMode) (f File, err error) {
-	if flag & ^(O_RDONLY|O_NOFOLLOW) != 0 {
-		return nil, pathError("open", name,
-			fmt.Errorf("invalid combination of flags 0x%x", flag))
-	}
-
-	fs.open.Do(func() {
-		f = newReaderFile(fs.ReadCloser, fs.fi(), fs.AllowEmptyFile)
-	})
-
-	if f == nil {
-		return nil, pathError("open", name, syscall.EIO)
-	}
-
-	return f, nil
-}
-
-// Stat returns a FileInfo describing the named file. If there is an error, it
-// will be of type *os.PathError.
-func (fs *Reader) Stat(name string) (os.FileInfo, error) {
-	return fs.Lstat(name)
-}
-
 // Lstat returns the FileInfo structure describing the named file.
 // If the file is a symbolic link, the returned FileInfo
 // describes the symbolic link.  Lstat makes no attempt to follow the link.
 // If there is an error, it will be of type *os.PathError.
-func (fs *Reader) Lstat(name string) (os.FileInfo, error) {
-	getDirInfo := func(name string) os.FileInfo {
-		fi := fakeFileInfo{
-			name:    fs.Base(name),
-			size:    0,
-			mode:    os.ModeDir | 0755,
-			modtime: time.Now(),
+func (fs *Reader) Lstat(name string) (*ExtendedFileInfo, error) {
+	getDirInfo := func(name string) *ExtendedFileInfo {
+		return &ExtendedFileInfo{
+			Name:    fs.Base(name),
+			Size:    0,
+			Mode:    os.ModeDir | 0755,
+			ModTime: time.Now(),
 		}
-		return fi
 	}
 
 	switch name {
@@ -184,13 +155,13 @@ func (fs *Reader) Dir(p string) string {
 	return path.Dir(p)
 }
 
-func newReaderFile(rd io.ReadCloser, fi os.FileInfo, allowEmptyFile bool) *readerFile {
+func newReaderFile(rd io.ReadCloser, fi *ExtendedFileInfo, allowEmptyFile bool) *readerFile {
 	return &readerFile{
 		ReadCloser:     rd,
 		AllowEmptyFile: allowEmptyFile,
 		fakeFile: fakeFile{
-			FileInfo: fi,
-			name:     fi.Name(),
+			fi:   fi,
+			name: fi.Name,
 		},
 	}
 }
@@ -232,28 +203,20 @@ var _ File = &readerFile{}
 // except Stat()
 type fakeFile struct {
 	name string
-	os.FileInfo
+	fi   *ExtendedFileInfo
 }
 
 // ensure that fakeFile implements File
 var _ File = fakeFile{}
 
-func (f fakeFile) Fd() uintptr {
-	return 0
+func (f fakeFile) MakeReadable() error {
+	return nil
 }
 
 func (f fakeFile) Readdirnames(_ int) ([]string, error) {
 	return nil, pathError("readdirnames", f.name, os.ErrInvalid)
 }
 
-func (f fakeFile) Readdir(_ int) ([]os.FileInfo, error) {
-	return nil, pathError("readdir", f.name, os.ErrInvalid)
-}
-
-func (f fakeFile) Seek(int64, int) (int64, error) {
-	return 0, pathError("seek", f.name, os.ErrInvalid)
-}
-
 func (f fakeFile) Read(_ []byte) (int, error) {
 	return 0, pathError("read", f.name, os.ErrInvalid)
 }
@@ -262,13 +225,24 @@ func (f fakeFile) Close() error {
 	return nil
 }
 
-func (f fakeFile) Stat() (os.FileInfo, error) {
-	return f.FileInfo, nil
+func (f fakeFile) Stat() (*ExtendedFileInfo, error) {
+	return f.fi, nil
+}
+
+func (f fakeFile) ToNode(_ bool) (*restic.Node, error) {
+	node := buildBasicNode(f.name, f.fi)
+
+	// fill minimal info with current values for uid, gid
+	node.UID = uint32(os.Getuid())
+	node.GID = uint32(os.Getgid())
+	node.ChangeTime = node.ModTime
+
+	return node, nil
 }
 
 // fakeDir implements Readdirnames and Readdir, everything else is delegated to fakeFile.
 type fakeDir struct {
-	entries []os.FileInfo
+	entries []string
 	fakeFile
 }
 
@@ -276,51 +250,7 @@ func (d fakeDir) Readdirnames(n int) ([]string, error) {
 	if n > 0 {
 		return nil, pathError("readdirnames", d.name, errors.New("not implemented"))
 	}
-	names := make([]string, 0, len(d.entries))
-	for _, entry := range d.entries {
-		names = append(names, entry.Name())
-	}
-
-	return names, nil
-}
-
-func (d fakeDir) Readdir(n int) ([]os.FileInfo, error) {
-	if n > 0 {
-		return nil, pathError("readdir", d.name, errors.New("not implemented"))
-	}
-	return d.entries, nil
-}
-
-// fakeFileInfo implements the bare minimum of os.FileInfo.
-type fakeFileInfo struct {
-	name    string
-	size    int64
-	mode    os.FileMode
-	modtime time.Time
-}
-
-func (fi fakeFileInfo) Name() string {
-	return fi.name
-}
-
-func (fi fakeFileInfo) Size() int64 {
-	return fi.size
-}
-
-func (fi fakeFileInfo) Mode() os.FileMode {
-	return fi.mode
-}
-
-func (fi fakeFileInfo) ModTime() time.Time {
-	return fi.modtime
-}
-
-func (fi fakeFileInfo) IsDir() bool {
-	return fi.mode&os.ModeDir > 0
-}
-
-func (fi fakeFileInfo) Sys() interface{} {
-	return nil
+	return slices.Clone(d.entries), nil
 }
 
 func pathError(op, name string, err error) *os.PathError {
