@@ -7,6 +7,6 @@ import "os"
 
 // OS-specific replacements of setFlags can set file status flags
 // that improve I/O performance.
-func setFlags(*os.File) error {
+func setFlags(_ *os.File) error {
 	return nil
 }
