@@ -28,9 +28,12 @@ type Restorer struct {
 
 	Error func(location string, err error) error
 	Warn  func(message string)
+	Info  func(message string)
 	// SelectFilter determines whether the item is selectedForRestore or whether a childMayBeSelected.
 	// selectedForRestore must not depend on isDir as `removeUnexpectedFiles` always passes false to isDir.
 	SelectFilter func(item string, isDir bool) (selectedForRestore bool, childMayBeSelected bool)
+
+	XattrSelectFilter func(xattrName string) (xattrSelectedForRestore bool)
 }
 
 var restorerAbortOnAllErrors = func(_ string, err error) error { return err }
@@ -69,7 +72,7 @@ func (c *OverwriteBehavior) Set(s string) error {
 		*c = OverwriteNever
 	default:
 		*c = OverwriteInvalid
-		return fmt.Errorf("invalid overwrite behavior %q, must be one of (always|if-newer|never)", s)
+		return fmt.Errorf("invalid overwrite behavior %q, must be one of (always|if-changed|if-newer|never)", s)
 	}
 
 	return nil
@@ -97,12 +100,13 @@ func (c *OverwriteBehavior) Type() string {
 // NewRestorer creates a restorer preloaded with the content from the snapshot id.
 func NewRestorer(repo restic.Repository, sn *restic.Snapshot, opts Options) *Restorer {
 	r := &Restorer{
-		repo:         repo,
-		opts:         opts,
-		fileList:     make(map[string]bool),
-		Error:        restorerAbortOnAllErrors,
-		SelectFilter: func(string, bool) (bool, bool) { return true, true },
-		sn:           sn,
+		repo:              repo,
+		opts:              opts,
+		fileList:          make(map[string]bool),
+		Error:             restorerAbortOnAllErrors,
+		SelectFilter:      func(string, bool) (bool, bool) { return true, true },
+		XattrSelectFilter: func(string) bool { return true },
+		sn:                sn,
 	}
 
 	return r
@@ -202,18 +206,18 @@ func (res *Restorer) traverseTreeInner(ctx context.Context, target, location str
 		}
 
 		// sockets cannot be restored
-		if node.Type == "socket" {
+		if node.Type == restic.NodeTypeSocket {
 			continue
 		}
 
-		selectedForRestore, childMayBeSelected := res.SelectFilter(nodeLocation, node.Type == "dir")
+		selectedForRestore, childMayBeSelected := res.SelectFilter(nodeLocation, node.Type == restic.NodeTypeDir)
 		debug.Log("SelectFilter returned %v %v for %q", selectedForRestore, childMayBeSelected, nodeLocation)
 
 		if selectedForRestore {
 			hasRestored = true
 		}
 
-		if node.Type == "dir" {
+		if node.Type == restic.NodeTypeDir {
 			if node.Subtree == nil {
 				return nil, hasRestored, errors.Errorf("Dir without subtree in tree %v", treeID.Str())
 			}
@@ -265,14 +269,14 @@ func (res *Restorer) traverseTreeInner(ctx context.Context, target, location str
 	return filenames, hasRestored, nil
 }
 
-func (res *Restorer) restoreNodeTo(ctx context.Context, node *restic.Node, target, location string) error {
+func (res *Restorer) restoreNodeTo(node *restic.Node, target, location string) error {
 	if !res.opts.DryRun {
 		debug.Log("restoreNode %v %v %v", node.Name, target, location)
 		if err := fs.Remove(target); err != nil && !errors.Is(err, os.ErrNotExist) {
 			return errors.Wrap(err, "RemoveNode")
 		}
 
-		err := node.CreateAt(ctx, target, res.repo)
+		err := fs.NodeCreateAt(node, target)
 		if err != nil {
 			debug.Log("node.CreateAt(%s) error %v", target, err)
 			return err
@@ -288,7 +292,7 @@ func (res *Restorer) restoreNodeMetadataTo(node *restic.Node, target, location s
 		return nil
 	}
 	debug.Log("restoreNodeMetadata %v %v %v", node.Name, target, location)
-	err := node.RestoreMetadata(target, res.Warn)
+	err := fs.NodeRestoreMetadata(node, target, res.Warn, res.XattrSelectFilter)
 	if err != nil {
 		debug.Log("node.RestoreMetadata(%s) error %v", target, err)
 	}
@@ -354,8 +358,9 @@ func (res *Restorer) RestoreTo(ctx context.Context, dst string) (uint64, error)
 
 	idx := NewHardlinkIndex[string]()
 	filerestorer := newFileRestorer(dst, res.repo.LoadBlobsFromPack, res.repo.LookupBlob,
-		res.repo.Connections(), res.opts.Sparse, res.opts.Delete, res.opts.Progress)
+		res.repo.Connections(), res.opts.Sparse, res.opts.Delete, res.repo.StartWarmup, res.opts.Progress)
 	filerestorer.Error = res.Error
+	filerestorer.Info = res.Info
 
 	debug.Log("first pass for %q", dst)
 
@@ -377,7 +382,7 @@ func (res *Restorer) RestoreTo(ctx context.Context, dst string) (uint64, error)
 				return err
 			}
 
-			if node.Type != "file" {
+			if node.Type != restic.NodeTypeFile {
 				res.opts.Progress.AddFile(0)
 				return nil
 			}
@@ -433,9 +438,9 @@ func (res *Restorer) RestoreTo(ctx context.Context, dst string) (uint64, error)
 	err = res.traverseTree(ctx, dst, *res.sn.Tree, treeVisitor{
 		visitNode: func(node *restic.Node, target, location string) error {
 			debug.Log("second pass, visitNode: restore node %q", location)
-			if node.Type != "file" {
+			if node.Type != restic.NodeTypeFile {
 				_, err := res.withOverwriteCheck(ctx, node, target, location, false, nil, func(_ bool, _ *fileState) error {
-					return res.restoreNodeTo(ctx, node, target, location)
+					return res.restoreNodeTo(node, target, location)
 				})
 				return err
 			}
@@ -511,12 +516,30 @@ func (res *Restorer) removeUnexpectedFiles(ctx context.Context, target, location
 		selectedForRestore, _ := res.SelectFilter(nodeLocation, false)
 		// only delete files that were selected for restore
 		if selectedForRestore {
-			res.opts.Progress.ReportDeletedFile(nodeLocation)
+			// First collect all files that will be deleted
+			var filesToDelete []string
+			err := filepath.Walk(nodeTarget, func(path string, _ os.FileInfo, err error) error {
+				if err != nil {
+					return err
+				}
+				filesToDelete = append(filesToDelete, path)
+				return nil
+			})
+			if err != nil {
+				return err
+			}
+
 			if !res.opts.DryRun {
+				// Perform the deletion
 				if err := fs.RemoveAll(nodeTarget); err != nil {
 					return err
 				}
 			}
+
+			// Report paths as deleted only after successful removal
+			for i := len(filesToDelete) - 1; i >= 0; i-- {
+				res.opts.Progress.ReportDeletion(filesToDelete[i])
+			}
 		}
 	}
 
@@ -547,7 +570,7 @@ func (res *Restorer) withOverwriteCheck(ctx context.Context, node *restic.Node,
 
 	var matches *fileState
 	updateMetadataOnly := false
-	if node.Type == "file" && !isHardlink {
+	if node.Type == restic.NodeTypeFile && !isHardlink {
 		// if a file fails to verify, then matches is nil which results in restoring from scratch
 		matches, buf, _ = res.verifyFile(ctx, target, node, false, res.opts.Overwrite == OverwriteIfChanged, buf)
 		// skip files that are already correct completely
@@ -616,7 +639,7 @@ func (res *Restorer) VerifyFiles(ctx context.Context, dst string, countRestoredF
 
 		err := res.traverseTree(ctx, dst, *res.sn.Tree, treeVisitor{
 			visitNode: func(node *restic.Node, target, location string) error {
-				if node.Type != "file" {
+				if node.Type != restic.NodeTypeFile {
 					return nil
 				}
 				if metadataOnly, ok := res.hasRestoredFile(location); !ok || metadataOnly {
